/*   StackFolder
 *   Copyright © 2012 ROSA  <support@rosalab.ru>
 *   License: GPLv2+
 *   Authors: Ural Mullabaev <ural.mullabaev@rosalab.ru>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public License
 *   along with this library; see the file COPYING.LIB.  If not, write to
 *   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *   Boston, MA 02110-1301, USA.
*/

#ifndef DIRECTORY_H
#define DIRECTORY_H

#include "file.h"

#include <KUrl>
#include <KFileItemList>

#include <QStringList>
#include <QTextStream>
#include <QDeclarativeListProperty>
#include <QObject>
#include <QKeyEvent>

#include <QBasicTimer>

class QAbstractItemModel;
class QModelIndex;
class ProxyModel;
class KFileItem;
class KFileItemList;

class DirModel;
class PreviewGenerator;

class Directory : public QObject {

    Q_OBJECT

    Q_PROPERTY(int filesCount READ filesCount)
    Q_PROPERTY(bool isTopUrl READ isTopUrl)
    Q_PROPERTY(QString path READ path)
    Q_PROPERTY(bool isDialogVisible READ isDialogVisible)
    Q_PROPERTY(bool isViewerRun READ isViewerRun)
    Q_PROPERTY(QDeclarativeListProperty<Directory> dirs READ dirs CONSTANT )
    Q_PROPERTY(QDeclarativeListProperty<File> files READ files CONSTANT )
    Q_PROPERTY(QString dirname READ dirname WRITE setDirname NOTIFY dirnameChanged)
    Q_PROPERTY(QString filename READ filename WRITE setFilename NOTIFY filenameChanged)
    Q_PROPERTY(QString fileContent READ fileContent WRITE setFileContent NOTIFY fileContentChanged)

    public:
        Directory(/*ProxyModel *model, KUrl &url,*/ QObject *parent = 0);

	void setModel(QAbstractItemModel *model);
	QAbstractItemModel *model() const;
	void setTopUrl(const KUrl &url);
	//KUrl &topUrl() const;
	void setUrl(const KUrl &url);
	KUrl &url() const;
        bool isTopUrl() const;

	QString path() const;

        bool isDialogVisible() const;
        bool isViewerRun() const;

        int dirsCount() const;
        int filesCount() const;
        QString dirname() const;
        QString filename() const;
        QString fileContent() const;
        QDeclarativeListProperty<Directory> dirs();
        QDeclarativeListProperty<File> files();

        void setDirname(const QString &str);
        void setFilename(const QString &str);
        void setFileContent(const QString &str);

	void setPreview(const KFileItem &item, const QPixmap &pixmap);
	void emitDialogShown();
	void emitDialogHidden();
	void emitKeyPressed(const QKeyEvent *event);

        Q_INVOKABLE void back();
        Q_INVOKABLE void activate(int index);
        Q_INVOKABLE void runViewer(int index, int x, int y, int width, int height);
        Q_INVOKABLE void stopViewer();
        Q_INVOKABLE void open();
        Q_INVOKABLE void activateDragAndDrop(int index);

    signals:
        void directoryChanged();
        void errorReceived(const QString &message);
        void dirnameChanged();
        void filenameChanged();
        void fileContentChanged();
        void fileActivated();
        void gotPreview(int i);
        void dataAdded(const QModelIndex &parent, int start, int end);
        void dialogShown();
        void dialogHidden();
        void keyPressed(const int key /*QKeyEvent *event*/, const int modifiers);
        void viewerRequested(const QString &path, int x, int y, int width, int height);
        void viewerCanceled();
        void activatedDragAndDrop(const KFileItem &file_item);

    public slots:
	void updatePreview();
        void viewerStopped();

    protected:
	void timerEvent(QTimerEvent *event);

    protected slots:
	void rowsInserted(const QModelIndex &parent, int start, int end);
	void listingStarted(const KUrl &url);
	void listingClear();
	void listingCompleted();
	void listingCanceled();
	void listingError(const QString &message);
	void listingDeleted(const KFileItemList &items);

    private:
	ProxyModel *m_model;
	DirModel *m_dirModel;
	KUrl m_url;
	KUrl m_topUrl;
        //QDir m_dir;
        QStringList m_dirDirs;
        QStringList m_dirFiles;
        File currentFile;

        QString m_fileContent;

        QList<Directory *> m_dirList;
        QList<File *> m_fileList;
        KFileItemList m_previewFileList;

	PreviewGenerator *m_previewGenerator;

	bool m_folderIsEmpty;
	QString m_errorMessage;

	bool m_directoryChanging;
	bool m_isDialogVisible;
	bool m_isViewerRun;

        QBasicTimer m_delayedPreviewTimer;
        int m_previewRow;

        void clear();
        void refresh();

};

#endif
